/*
 *
 *  Copyright (C) 2004 Aurelien Alleaume <slts@free.fr>
 *  Copyright (C) 2005 Mike Isely <isely@pobox.com>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "pvrusb2.h"
#include "pvrusb2-v4l2.h"
#include "saa7115.h"
#include "ivtv.h"
#include <linux/video_decoder.h>

int pvr_decoder_set_input(struct pvr *pvr)
{
    int v = 0;
#ifdef notdef
    pvr_debug("pvr_decoder_set_input(%d)",pvr->video.input);
#endif
    switch(pvr->video.input){
    case PVR_INPUT_TV:
	v = 4;
	break;
    case PVR_INPUT_COMPOSITE:
	v = 5;
	break;
    case PVR_INPUT_SVIDEO:
	v = 8;
	break;
    default:
	return -EINVAL;
    }
    return pvr_i2c_saa7115_cmd(pvr,DECODER_SET_INPUT,&v);
}

int pvr_decoder_set_bcsh(struct pvr *pvr)
{
    struct saa7115_state st;
    int ret;
#ifdef notdef
    pvr_debug("pvr_decoder_set_bcsh b=%d c=%d s=%d h=%d",
	      pvr->video.brightness,
	      pvr->video.contrast,
	      pvr->video.saturation,
	      pvr->video.hue);
#endif
    st.bright = pvr->video.brightness;
    st.contrast = pvr->video.contrast;
    st.sat = pvr->video.saturation;
    st.hue = pvr->video.hue;
    ret = pvr_i2c_saa7115_cmd(pvr,DECODER_SET_PICTURE,&st);
    return ret;
}

int pvr_decoder_is_tuned(struct pvr *pvr)
{
    int status;
    int ret;
    ret = pvr_i2c_saa7115_cmd(pvr,DECODER_GET_STATUS,&status);
    if (ret < 0) return 0;
    if (status & DECODER_STATUS_GOOD) return 1;
    return 0;
}

int pvr_decoder_set_size(struct pvr *pvr)
{
    struct video_window wind;
    wind.width = pvr->video.width;
    wind.height = ((pvr->video.standard == PVR_STD_NTSC) ? 480 : 576);
#ifdef notdef
    pvr_debug("pvr_decoder_set_set %dx%d",wind.width,wind.height);
#endif
    return pvr_i2c_saa7115_cmd(pvr,DECODER_SET_SIZE,&wind);
}

int pvr_decoder_set_audio(struct pvr *pvr)
{
    int val;
#ifdef notdef
    pvr_debug("pvr_decoder_set_audio %d",pvr->audio.srate);
#endif
    switch (pvr->audio.srate) {
    default:
    case PVR_SRATE_48:
	val = DECODER_AUDIO_48_KHZ;
	break;
    case PVR_SRATE_44:
	val = DECODER_AUDIO_441_KHZ;
	break;
    }
    return pvr_i2c_saa7115_cmd(pvr,DECODER_SET_AUDIO,&val);
}

int pvr_decoder_set_norm(struct pvr *pvr)
{
    int val;
#ifdef notdef
    pvr_debug("pvr_decoder_set_norm %d",pvr->video.standard);
#endif
    switch (pvr->video.standard) {
    default:
    case PVR_STD_NTSC:
	val = VIDEO_MODE_NTSC;
	break;
    case PVR_STD_SECAM:
	val = VIDEO_MODE_SECAM;
	break;
    case PVR_STD_PAL_I:
    case PVR_STD_PAL_DK:
    case PVR_STD_PAL_BG:
	val = VIDEO_MODE_PAL;
	break;
    }
    return pvr_i2c_saa7115_cmd(pvr,DECODER_SET_NORM,&val);
}

int pvr_decoder_set_vbi(struct pvr *pvr)
{
    int val = 0;
#ifdef notdef
    pvr_debug("pvr_decoder_set_vbi %d",pvr->config == _vbi);
#endif
    if (pvr->config == _vbi) {
	if (pvr->video.standard == PVR_STD_NTSC) {
	    val = IVTV_SLICED_CAPTION_525;
	} else {
	    val = IVTV_SLICED_CAPTION_625;
	}
    }
    return pvr_i2c_saa7115_cmd(pvr,DECODER_SET_VBI,(void *)val);
}

int pvr_configure_decoder(struct pvr *pvr)
{
    int ret;
    ret = pvr_decoder_set_norm(pvr);
    if (ret < 0) return ret;
    ret = pvr_decoder_set_size(pvr);
    if (ret < 0) return ret;
    ret = pvr_decoder_set_bcsh(pvr);
    if (ret < 0) return ret;
    ret = pvr_decoder_set_audio(pvr);
    if (ret < 0) return ret;
#ifdef notdef
    // This causes a null pointer crash inside of saa7115.ko
    ret = pvr_decoder_set_vbi(pvr);
    if (ret < 0) return ret;
#endif
    return 0;
}
